@@ -9,6 +9,7 @@ from numbers import Integral, Real
 import numpy as np
 from tlz import concat, interleave, sliding_window
 
+from dask._task_spec import Task, TaskRef
 from dask.array import chunk
 from dask.array.core import (
     Array,
@@ -28,10 +29,10 @@ from dask.array.core import (
     tensordot_lookup,
 )
 from dask.array.creation import arange, diag, empty, indices, tri
-from dask.array.einsumfuncs import einsum  # noqa
+from dask.array.einsumfuncs import einsum  # noqa: F401
 from dask.array.numpy_compat import NUMPY_GE_200
 from dask.array.reductions import reduction
-from dask.array.ufunc import multiply, sqrt
+from dask.array.ufunc import multiply, sqrt, true_divide
 from dask.array.utils import (
     array_safe,
     asarray_safe,
@@ -525,7 +526,7 @@ def apply_along_axis(func1d, axis, arr, *args, dtype=None, shape=None, **kwargs)
     # Adds other axes as needed.
     result = arr.map_blocks(
         _inner_apply_along_axis,
-        name=funcname(func1d) + "-along-axis",
+        token=funcname(func1d) + "-along-axis",
         dtype=dtype,
         chunks=(arr.chunks[:axis] + shape + arr.chunks[axis + 1 :]),
         drop_axis=axis,
@@ -849,23 +850,13 @@ def searchsorted(a, v, side="left", sorter=None):
     return out
 
 
-# TODO: dask linspace doesn't support delayed values
-def _linspace_from_delayed(start, stop, num=50):
-    linspace_name = "linspace-" + tokenize(start, stop, num)
-    (start_ref, stop_ref, num_ref), deps = unpack_collections([start, stop, num])
-    if len(deps) == 0:
-        return np.linspace(start, stop, num=num)
-
-    linspace_dsk = {(linspace_name, 0): (np.linspace, start_ref, stop_ref, num_ref)}
-    linspace_graph = HighLevelGraph.from_collections(
-        linspace_name, linspace_dsk, dependencies=deps
-    )
-
-    chunks = ((np.nan,),) if is_dask_collection(num) else ((num,),)
-    return Array(linspace_graph, linspace_name, chunks, dtype=float)
+def _linspace(bins_range):
+    bins, (start, stop) = bins_range
+    return np.linspace(start, stop, num=bins + 1)
 
 
-def _block_hist(x, bins, range=None, weights=None):
+def _block_hist(x, bins_range, weights=None):
+    bins, range = bins_range
     return np.histogram(x, bins, range=range, weights=weights)[0][np.newaxis]
 
 
@@ -1006,10 +997,27 @@ def histogram(a, bins=None, range=None, normed=False, weights=None, density=None
     token = tokenize(a, bins, range, weights, density)
     name = "histogram-sum-" + token
 
+    bins_range, deps = unpack_collections([bins, range])
+
     if scalar_bins:
-        bins = _linspace_from_delayed(range[0], range[1], bins + 1)
-        # ^ NOTE `range[1]` is safe because of the above check, and the initial check
-        # that range must not be None if `scalar_bins`
+        assert range is not None
+        assert bins is not None
+        if len(deps) == 0:
+            bins = np.linspace(range[0], range[1], num=bins + 1)
+        else:
+            linspace_name = "linspace-" + tokenize(bins_range)
+
+            linspace_dsk = {
+                (linspace_name, 0): Task((linspace_name, 0), _linspace, bins_range)
+            }
+            linspace_graph = HighLevelGraph.from_collections(
+                linspace_name, linspace_dsk, dependencies=deps
+            )
+            if is_dask_collection(bins):
+                chunks = ((np.nan,),)
+            else:
+                chunks = ((bins + 1,),)
+            bins = Array(linspace_graph, linspace_name, chunks, dtype=float)
     else:
         if not isinstance(bins, (Array, np.ndarray)):
             bins = asarray(bins)
@@ -1018,12 +1026,10 @@ def histogram(a, bins=None, range=None, normed=False, weights=None, density=None
                 f"bins must be a 1-dimensional array or sequence, got shape {bins.shape}"
             )
 
-    (bins_ref, range_ref), deps = unpack_collections([bins, range])
-
     # Map the histogram to all bins, forming a 2D array of histograms, stacked for each chunk
     if weights is None:
         dsk = {
-            (name, i, 0): (_block_hist, k, bins_ref, range_ref)
+            (name, i, 0): Task((name, i, 0), _block_hist, TaskRef(k), bins_range)
             for i, k in enumerate(flatten(a.__dask_keys__()))
         }
         dtype = np.histogram([])[0].dtype
@@ -1031,7 +1037,9 @@ def histogram(a, bins=None, range=None, normed=False, weights=None, density=None
         a_keys = flatten(a.__dask_keys__())
         w_keys = flatten(weights.__dask_keys__())
         dsk = {
-            (name, i, 0): (_block_hist, k, bins_ref, range_ref, w)
+            (name, i, 0): Task(
+                (name, i, 0), _block_hist, TaskRef(k), bins_range, TaskRef(w)
+            )
             for i, (k, w) in enumerate(zip(a_keys, w_keys))
         }
         dtype = weights.dtype
@@ -1354,10 +1362,8 @@ def histogramdd(sample, bins, range=None, normed=None, weights=None, density=Non
     # range= these are unsupported.
     dc_bins = is_dask_collection(bins)
     if isinstance(bins, (list, tuple)):
-        dc_bins = dc_bins or any([is_dask_collection(b) for b in bins])
-    dc_range = (
-        any([is_dask_collection(r) for r in range]) if range is not None else False
-    )
+        dc_bins = dc_bins or any(is_dask_collection(b) for b in bins)
+    dc_range = any(is_dask_collection(r) for r in range) if range is not None else False
     if dc_bins or dc_range:
         raise NotImplementedError(
             "Passing dask collections to bins=... or range=... is not supported."
@@ -1516,7 +1522,17 @@ def histogramdd(sample, bins, range=None, normed=None, weights=None, density=Non
 
 
 @derived_from(np)
-def cov(m, y=None, rowvar=1, bias=0, ddof=None):
+def cov(
+    m,
+    y=None,
+    rowvar=True,
+    bias=False,
+    ddof=None,
+    fweights=None,
+    aweights=None,
+    *,
+    dtype=None,
+):
     # This was copied almost verbatim from np.cov
     # See numpy license at https://github.com/numpy/numpy/blob/master/LICENSE.txt
     # or NUMPY_LICENSE.txt within this directory
@@ -1525,47 +1541,87 @@ def cov(m, y=None, rowvar=1, bias=0, ddof=None):
 
     # Handles complex arrays too
     m = asarray(m)
+
     if y is None:
-        dtype = np.result_type(m, np.float64)
+        dtype = result_type(m, np.float64)
     else:
         y = asarray(y)
-        dtype = np.result_type(m, y, np.float64)
+        dtype = result_type(m, y, np.float64)
+
+    if m.ndim > 2:
+        raise ValueError("m has more than 2 dimensions")
+    if y is not None and y.ndim > 2:
+        raise ValueError("y has more than 2 dimensions")
+
     X = array(m, ndmin=2, dtype=dtype)
 
-    if X.shape[0] == 1:
-        rowvar = 1
-    if rowvar:
-        N = X.shape[1]
-        axis = 0
-    else:
-        N = X.shape[0]
-        axis = 1
-
-    # check ddof
     if ddof is None:
-        if bias == 0:
-            ddof = 1
+        ddof = 1 if bias == 0 else 0
+
+    if not rowvar and m.ndim != 1:
+        X = X.T
+    if X.shape[0] == 0:
+        return array([]).reshape(0, 0)
+    if y is not None:
+        y = array(y, ndmin=2, dtype=dtype)
+        if not rowvar and y.shape[0] != 1:
+            y = y.T
+        X = concatenate((X, y), axis=0)
+
+    # Unlike NumPy, these checks don't include:
+    #    - if fweights are all integers
+    #    - if either fweights or aweights are all non-negative
+    # These checks potientially expensive for distributed arrays.
+    w = None
+    if fweights is not None:
+        fweights = asarray(fweights, dtype=float)
+        if fweights.ndim > 1:
+            raise RuntimeError("cannot handle multidimensional fweights")
+        if fweights.shape[0] != X.shape[1]:
+            raise RuntimeError("incompatible numbers of samples and fweights")
+        w = fweights
+    if aweights is not None:
+        aweights = asarray(aweights, dtype=float)
+        if aweights.ndim > 1:
+            raise RuntimeError("cannot handle multidimensional aweights")
+        if aweights.shape[0] != X.shape[1]:
+            raise RuntimeError("incompatible numbers of samples and aweights")
+        if w is None:
+            w = aweights
         else:
-            ddof = 0
-    fact = float(N - ddof)
+            w *= aweights
+
+    avg, w_sum = average(X, axis=1, weights=w, returned=True)
+    w_sum = w_sum[0]
+
+    # Determine the normalization
+    if w is None:
+        fact = X.shape[1] - ddof
+    elif ddof == 0:
+        fact = w_sum
+    elif aweights is None:
+        fact = w_sum - ddof
+    else:
+        fact = w_sum - ddof * sum(w * aweights) / w_sum
+
     if fact <= 0:
         warnings.warn("Degrees of freedom <= 0 for slice", RuntimeWarning)
         fact = 0.0
 
-    if y is not None:
-        y = array(y, ndmin=2, dtype=dtype)
-        X = concatenate((X, y), axis)
-
-    X = X - X.mean(axis=1 - axis, keepdims=True)
-    if not rowvar:
-        return (dot(X.T, X.conj()) / fact).squeeze()
+    X -= avg[:, None]
+    if w is None:
+        X_T = X.T
     else:
-        return (dot(X, X.T.conj()) / fact).squeeze()
+        X_T = (X * w).T
+    c = dot(X, X_T.conj())
+    c *= true_divide(1, fact)
+    return c.squeeze()
 
 
 @derived_from(np)
 def corrcoef(x, y=None, rowvar=1):
     c = cov(x, y, rowvar)
+
     if c.shape == ():
         return c / c
     d = diag(c)
@@ -1686,9 +1742,11 @@ def unique_no_structured_arr(
         (name, 0): (
             (np.unique,)
             + tuple(
-                (np.concatenate, o.__dask_keys__())
-                if hasattr(o, "__dask_keys__")
-                else o
+                (
+                    (np.concatenate, o.__dask_keys__())
+                    if hasattr(o, "__dask_keys__")
+                    else o
+                )
                 for o in out_parts
             )
         )
@@ -1773,9 +1831,11 @@ def unique(ar, return_index=False, return_inverse=False, return_counts=False):
         (name, 0): (
             (_unique_internal,)
             + tuple(
-                (np.concatenate, o.__dask_keys__())
-                if hasattr(o, "__dask_keys__")
-                else o
+                (
+                    (np.concatenate, o.__dask_keys__())
+                    if hasattr(o, "__dask_keys__")
+                    else o
+                )
                 for o in out_parts
             )
             + (return_inverse,)
@@ -2026,7 +2086,7 @@ def _asarray_isnull(values):
 def isnull(values):
     """pandas.isnull for dask arrays"""
     # eagerly raise ImportError, if pandas isn't available
-    import pandas as pd  # noqa
+    import pandas as pd  # noqa: F401
 
     return elemwise(_asarray_isnull, values, dtype="bool")
 
@@ -2200,7 +2260,7 @@ def piecewise(x, condlist, funclist, *args, **kw):
         x,
         *condlist,
         dtype=x.dtype,
-        name="piecewise",
+        token="piecewise",
         funclist=funclist,
         func_args=args,
         func_kw=kw,
@@ -2419,11 +2479,16 @@ def delete(arr, obj, axis):
     target_arr = split_at_breaks(arr, obj, axis)
 
     target_arr = [
-        arr[
-            tuple(slice(1, None) if axis == n else slice(None) for n in range(arr.ndim))
-        ]
-        if i != 0
-        else arr
+        (
+            arr[
+                tuple(
+                    slice(1, None) if axis == n else slice(None)
+                    for n in range(arr.ndim)
+                )
+            ]
+            if i != 0
+            else arr
+        )
         for i, arr in enumerate(target_arr)
     ]
     return concatenate(target_arr, axis=axis)
